# coding: utf-8
from __future__ import absolute_import
from __future__ import unicode_literals

from django.conf import settings
import six
import json

from m3_auth.config import Config
from m3_auth.proxy import SessionProxy
from m3_auth.utils import SingletonMeta
from m3_auth.utils import import_object
from m3_auth.utils import is_authenticated
from m3_auth.utils import get_request_params
from m3_auth.exceptions import BadRequest


class AuthManager(six.with_metaclass(SingletonMeta)):

    """Менеджер - инкапсулирует логику работы с разными бэкендами.

    :ivar backends: Словарь бэкендов
    :type backends: dict of auth.backends.base.BaseBackend
    """

    def __init__(self):
        self.backends = {}

        if getattr(settings, 'AUTH_CONFIG_FILE', None):
            self._load_backends()

    def get_backend(self, request):
        """Возвращает бэкенд на основе HTTP-запроса.

        Название бэкенда находится либо в сессии пользователя, либо в
        параметрах запроса.

        Если пользователь аутентифицирован, то название бэкенда берется только
        из сессии. Если пользователь не аутентифицирован, название бэкенда
        берется либо из параметров запроса, либо из сессии если это повторный
        запрос.

        :param request: HTTP-запрос.

        :raises: auth.exceptions.BadRequest

        :return: Объект класса бэкенда.
        :rtype: auth.backends.base.BaseBackend
        """
        session_proxy = SessionProxy(request)
        params = get_request_params(request)
        name = session_proxy.backend

        if not is_authenticated(request):
            name = params.get('backend') or session_proxy.backend
            session_proxy.backend = name

        if name not in self.backends:
            raise BadRequest()

        return self.backends[name]

    def get_login_url(self, request):
        """Возвращает URL, полученный от бэкенда для аутентификации.

        :param request: HTTP-запрос.
        :type request: django.http.HttpRequest

        :return: URL адрес.
        :rtype: str
        """
        return self.get_backend(request).get_login_url(request)

    def process_login(self, request):
        """Возвращает результат обработки запроса аутентификации.

        :param request: Запрос.
        :type request: django.http.HttpRequest

        :return: Пользователь, под которым осуществлен вход в систему.
        :rtype: django.contrib.auth.models.User
        """
        return self.get_backend(request).process_login(request)

    def get_logout_url(self, request):
        """Возвращает URL адрес для выхода из Системы.

        :param request: HTTP-запрос.
        :type request: django.http.HttpRequest

        :return: URL адрес.
        :rtype: str
        """
        return self.get_backend(request).get_logout_url(request)

    def process_logout(self, request):
        """Возвращает результат обработки запроса на выход из Системы.

        :param request: HTTP-запрос.
        :type request: django.http.HttpRequest

        :return: True или False.
        :rtype: bool
        """
        return self.get_backend(request).process_logout(request)

    def __getattr__(self, method_name):
        """Если метод не определен, то переадресовывает вызов бэкенду.

        :param method_name: Наименование метода.
        :return: Функция для вызова.
        """
        def wrapper(request, *args, **kwargs):
            backend = self.get_backend(request)
            method = getattr(backend, method_name)

            if not callable(method):
                raise BadRequest()

            return method(request, *args, **kwargs)
        return wrapper

    def register_backend(self, config):
        """Регистрирует в менеджере бэкенд аутентификации.

        :param dict config: конфигурация бэкенда.
        """
        config = Config(**config)
        backend_cls = import_object(config.backend_cls)
        backend = backend_cls.from_config(config)
        self.backends[backend.name] = backend

    def _load_backends(self):
        """Метод собирает и настраивает бэкенды из настроек."""
        with open(settings.AUTH_CONFIG_FILE) as fs:
            configs = json.loads(fs.read())

        for config in configs:
            self.register_backend(config)


manager = AuthManager()
