# coding: utf-8

from __future__ import absolute_import
from __future__ import unicode_literals

from importlib import import_module

from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.auth.middleware import AuthenticationMiddleware
from django.contrib.sessions.middleware import SessionMiddleware
from django.core.cache import cache
from django.http.response import HttpResponse
from django.test.client import RequestFactory
from django.test.testcases import TestCase

from m3_auth.backends.base.remote.protocol import BaseProtocol
from m3_auth.backends.base.remote.user import BaseRemoteUser
from m3_auth.backends.saml2 import SAMLBackend
from m3_auth.utils import is_authenticated
from m3_auth.config import Config


class TestSAMLProtocol(BaseProtocol):

    """Тестовый протокол, для проверки SAMLBackend'а."""

    def process_single_logout(self, request):
        return 1, 'some_url'

    def get_logout_url(self, request):
        pass

    def process_logout(self, request):
        pass

    def process_login(self, request):
        pass

    def get_login_url(self, request):
        pass

    @classmethod
    def from_config(cls, config):
        pass


class TestSAMLUser(BaseRemoteUser):

    """Тестовый SAML пользователь."""


class TestSAMLBackend(TestCase):

    """Проверка SAML бэкенда."""

    def setUp(self):
        super(TestSAMLBackend, self).setUp()
        self.factory = RequestFactory()
        self.user = User.objects.get(username='admin')

        protocol = TestSAMLProtocol(TestSAMLUser, Config())
        self.backend = SAMLBackend('saml', protocol, [])
        self.session_store = import_module(
            settings.SESSION_ENGINE).SessionStore()

    def test_process_single_logout(self):
        """Проверка метода process_single_logout."""
        request = self.factory.post('/')
        self.process_request(request)

        # Входим
        self.backend._login(request, self.user, 1)
        self.process_response(request)

        cache_key = self.backend._get_cache_key(1)
        session_key = cache.get(cache_key)
        self.assertTrue(self.session_store.exists(session_key))

        # Выходят за нас
        response_url = self.backend.process_single_logout(request)

        self.assertFalse(cache.get(cache_key))
        self.assertEqual(response_url, 'some_url')
        self.assertFalse(self.session_store.exists(session_key))

    def test_login_and_force_logout(self):
        """Проверка метода _login."""
        request = self.factory.post('/')
        self.process_request(request)

        # Входим
        self.backend._login(request, self.user, 1)
        self.process_response(request)

        cache_key = self.backend._get_cache_key(1)
        session_key = cache.get(cache_key)

        self.assertEqual(session_key, request.session.session_key)
        self.assertTrue(is_authenticated(request))
        self.assertTrue(self.session_store.exists(session_key))

        # Выходим
        self.backend._force_logout(1)

        self.assertFalse(cache.get(cache_key))
        self.assertFalse(self.session_store.exists(session_key))

    def test_get_cache_key(self):
        """Проверка метода _get_cache_key."""
        cache_key = self.backend._get_cache_key(15)
        self.assertEqual(cache_key, '{}:user_id:15'.format(self.backend.name))

    @staticmethod
    def process_request(request):
        SessionMiddleware().process_request(request)
        AuthenticationMiddleware().process_request(request)

    @staticmethod
    def process_response(request):
        SessionMiddleware().process_response(request, HttpResponse())
